@@ -59,7 +59,7 @@
 #include <linux/string.h>
 //#include <asm/spinlock.h>
 //#include <asm/bitops.h>
-//#include <asm/io.h>
+#include <asm/io.h>
 //#include <asm/irq.h>
 
 #ifdef IANS
@@ -130,6 +130,12 @@
 #define RX_ALIGN_ERRORS_TAG            	"Rx_Align_Errors"
 #define RX_LONG_LENGTH_ERRORS_TAG       "Rx_Long_Length_Errors"
 
+#define TX_FCTL_PACKETS_TAG             "Tx_Flow_Control_Pause"
+#define RX_FCTL_PACKETS_TAG             "Rx_Flow_Control_Pause"
+#define RX_FCTL_UNSUPPOORTED_TAG        "Rx_Flow_Control_Unsup"
+
+#define TX_TCO_PACKETS_TAG              "Tx_TCO_Packets"
+#define RX_TCO_PACKETS_TAG              "Rx_TCO_Packets"
 
 /* E100 compile time configurable parameters */
 
@@ -176,7 +182,15 @@
 #define E100_MAX_RFD       1024
 #endif /*  __ia64__ */
 
-#define E100_DEFAULT_XSUM  TRUE
+#define E100_DEFAULT_XSUM         TRUE
+#define E100_DEFAULT_BER          ZLOCK_MAX_ERRORS
+#define E100_DEFAULT_SPEED_DUPLEX 0
+
+#ifdef __ia64__
+#define E100_DEFAULT_UCODE        FALSE
+#else
+#define E100_DEFAULT_UCODE        TRUE
+#endif /*  __ia64__ */
 
 #define TX_CONG_DFLT   0       /* congestion enable flag for National TX Phy */
 #define TX_FIFO_LMT    0
@@ -192,7 +206,9 @@
 #define CB_557_CFIG_DEFAULT_PARM4   0x00
 #define CB_557_CFIG_DEFAULT_PARM5   0x00
 #define DEFAULT_TRANSMIT_THRESHOLD     12   /* 12 -> 96 bytes */
-#define ADAPTIVE_IFS                0
+/* IFS parameters */
+#define MIN_NUMBER_OF_TRANSMITS_100 1000
+#define MIN_NUMBER_OF_TRANSMITS_10  100
 
 /* Note: The 2 parameters below might affect NFS performance. If there are
  * problems with NFS, set CPU_CYCLE_SAVER to 0x0 and CFG_BYTE_PARM6 to 0x32.
@@ -262,6 +278,8 @@
  */
 #if   ( LINUX_VERSION_CODE < KERNEL_VERSION(2,3,23) )
 #define pci_enable_device(dev) do{} while(0)
+#define __constant_cpu_to_le32 cpu_to_le32
+#define __constant_cpu_to_le16 cpu_to_le16
 #endif
 
 /* Dynamic DMA maping
@@ -308,6 +326,10 @@ static inline int request_mem_region(long int addr, ...) { return 1; }
 static inline int release_mem_region(long int addr, ...) { return 0; }
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
+struct pci_device_id {};
+#endif
+
 /*****************************************************************************
  **
  **  Network Device API changes
@@ -395,6 +417,11 @@ typedef struct _err_stats_t
     uint32_t rcv_runts;
     uint32_t num_gd_xmts;           /* # of last good tx frames since the last
                                        adjustment of the TX THRESHOLD */
+    uint32_t xmt_fc_pkts;
+    uint32_t rcv_fc_pkts;
+    uint32_t rcv_fc_unsupported;
+    uint32_t xmt_tco_pkts;
+    uint32_t rcv_tco_pkts;
 } err_stats_t, *perr_stats_t;
 
 #define STATIC static
@@ -416,7 +443,7 @@ typedef struct net_device_stats net_device_stats_t;
 #define PRINT_SPEED_DPLX      0x200
 #define BOARD_IN_SERVICE      0x400
 #define IS_ICH                0x800
-#define IS_82562EH            0x1000
+
 
 /* Changed for 82558 and 82559 enhancements */
 /* defines for 82558/9 flow control configure paramters */
@@ -662,8 +689,11 @@ typedef struct net_device_stats net_device_stats_t;
 #define EEPROM_SUBSYSTEM_ID_WORD        0x0b
 #define EEPROM_SUBSYSTEM_VENDOR_WORD    0x0c
 
-#define E100_WRITE_REG(reg, value) {bddp->scbp->reg = value; }
-#define E100_READ_REG(reg) bddp->scbp->reg
+
+/*
+#define E100_WRITE_REG(reg, value) {bddp->scbp->(reg) = value; }
+#define E100_READ_REG(reg) bddp->scbp->(reg)
+*/
 
 #define EEPROM_CTRL scb_eprm_cntrl
 #define EEPROM_CHECKSUM_REG             0x3f
@@ -759,6 +789,7 @@ typedef struct net_device_stats net_device_stats_t;
 #define CB_CFIG_LATE_SCB           BIT_0   /* Update SCB After New Tx Start */
 #define CB_CFIG_DIRECT_DMA_DIS     BIT_1   /* Direct DMA mode */
 #define CB_CFIG_TNO_INT            BIT_2   /* Tx Not OK Interrupt */
+#define CB_CFIG_TCO_STAT           BIT_2   /* TCO statistics in 559 and above*/
 #define CB_CFIG_CI_INT             BIT_3   /* Command Complete Interrupt */
 #define CB_CFIG_EXT_TCB_DIS        BIT_4   /* Extended TCB */
 #define CB_CFIG_EXT_STAT_DIS       BIT_5   /* Extended Stats */
@@ -1418,7 +1449,7 @@ typedef struct net_device_stats net_dev_stats_t;
 
 #define E100_MIN(X,Y)         ( (X) < (Y)? (X):(Y) )
 #define RFD_POINTER(X,Y)      ((rfd_t *) (((unsigned char *)((X)->data))-((Y)->rfd_size)))
-#define SKB_RFD_COMPLETE(X,Y) ((RFD_POINTER((X),(Y)))->rfd_header.cb_status)
+#define SKB_RFD_STATUS(X,Y) ((RFD_POINTER((X),(Y)))->rfd_header.cb_status)
 #define NEXT_SKB(X,Y)         ((RFD_POINTER((X),(Y)))->next)   
 #define GET_SKB_DMA_ADDR(skb,bddp) ((RFD_POINTER(skb,bddp))->dma_addr)
 /* ====================================================================== */
@@ -1568,9 +1599,11 @@ typedef struct _self_test_t
 } self_test_t, *pself_test_t __attribute__ ((__packed__));
 
 
-
-/* Error Counters */
-typedef struct _err_cntr_t
+/* 
+ *  Statistical Counters 
+ */
+/* 82557 counters */
+typedef struct _basic_cntr_t
 {
     uint32_t xmt_gd_frames;        /* Good frames transmitted */
     uint32_t xmt_max_coll;        /* Fatal frames -- had max collisions */
@@ -1588,16 +1621,76 @@ typedef struct _err_cntr_t
     uint32_t rcv_oruns;           /* Overrun errors - bus was busy */
     uint32_t rcv_err_coll;        /* Received frms. that encountered coll. */
     uint32_t rcv_shrt_frames;     /* Received frames that were to short */
-    uint32_t cmd_complete;        /* A005h indicates cmd completion */
-} err_cntr_t, *perr_cntr_t;
+} basic_cntr_t, *pbasic_cntr_t;
+
+
+/* 82558 extended statistic counters */
+typedef struct _ext_cntr_t
+{
+    uint32_t xmt_fc_frames;
+    uint32_t rcv_fc_frames;
+    uint32_t rcv_fc_unsupported;           
+} ext_cntr_t, *pext_cntr_t;
+
+
+/* 82559 TCO statistic counters */
+typedef struct _tco_cntr_t
+{
+    uint16_t xmt_tco_frames;
+    uint16_t rcv_tco_frames;
+} tco_cntr_t, *ptco_cntr_t;
+
+/* Structures to access thet physical dump area */
+/* Use one of these types, according to the statisitcal counters mode,
+   to cast the pointer to the physical dump area and access the cmd_complete
+   DWORD. */
+
+/* 557-mode : only basic counters + cmd_complete */
+typedef struct _err_cntr_557_t 
+{
+	basic_cntr_t	basic_stats;
+	uint32_t		cmd_complete;
+} err_cntr_557_t, *perr_cntr_557_t;
+
+/* 558-mode : basic + extended counters + cmd_complete */
+typedef struct _err_cntr_558_t 
+{
+	basic_cntr_t	basic_stats;	
+	ext_cntr_t		extended_stats;
+	uint32_t		cmd_complete;
+} err_cntr_558_t, *perr_cntr_558_t;
+
+/* 559-mode : basic + extended + TCO counters + cmd_complete */
+typedef struct _err_cntr_559_t 
+{
+	basic_cntr_t	basic_stats;
+	ext_cntr_t		extended_stats;
+	tco_cntr_t		tco_stats;
+	uint32_t		cmd_complete;
+} err_cntr_559_t, *perr_cntr_559_t;
+
+/* This typedef defines the struct needed to hold the largest number of counters */
+typedef err_cntr_559_t max_counters_t, *pmax_counters_t;
+
+
+/* Different statistical-counters mode the controller may be in */
+typedef enum _stat_mode_t {
+	E100_BASIC_STATS = 0,	/* 82557 stats : 16 counters / 16 dw */
+	E100_EXTENDED_STATS,	/* 82558 stats : 19 counters / 19 dw */  
+	E100_TCO_STATS			/* 82559 stats : 21 counters / 20 dw */
+} stat_mode_t, *pstat_mode_t;
+
+/* dump statistical counters complete codes */
+#define DUMP_STAT_COMPLETE_CODE			0xA005
+#define DUMP_RST_STAT_COMPLETE_CODE		0xA007
 
 
 /* Command Block (CB) Generic Header Structure*/
 typedef struct _cb_header_t
 {
-    uint16_t cb_status;              /* Command Block Status */
-    uint16_t cb_cmd;                 /* Command Block Command */
-    uint32_t cb_lnk_ptr;           /* Link To Next CB */
+    volatile uint16_t cb_status;              /* Command Block Status */
+    volatile uint16_t cb_cmd;                 /* Command Block Command */
+    volatile uint32_t cb_lnk_ptr;           /* Link To Next CB */
 } cb_header_t, *pcb_header_t __attribute__ ((__packed__));
 
 
@@ -1745,7 +1838,7 @@ struct _tcb_t
   sk_buff_t *tcb_skb;           /* the associated socket buffer */
   int tcb_cbflag;              /* set if using the coalesce buffer */
   int tcb_msgsz;
-  uint32_t dma_data_addr;
+  dma_addr_t dma_data_addr;
   uint32_t tcb_paddr;           /* phys addr of the TCB */
 } __attribute__ ((__packed__));
 
@@ -1832,8 +1925,9 @@ typedef struct _bdd_t
     pscb_t scbp;                 /* memory mapped ptr to 82557 scb */
     pself_test_t pselftest;        /* pointer to self test area */
     dma_addr_t selftest_paddr;        /* phys addr of selftest */
-    perr_cntr_t pstats_counters; /* pointer to stats table */
+    pmax_counters_t pstats_counters; /* pointer to stats table */
     dma_addr_t stat_cnt_paddr;        /* phys addr of stat counter area */
+    stat_mode_t stat_mode;
     pdump_area_t pdump_area;     /* pointer to 82557 reg. dump area */
     dma_addr_t dump_paddr;           /* phys addr of dump area */
     pnxmit_cb_t pntcb;           /* pointer to non xmit tcb */
@@ -1903,11 +1997,6 @@ typedef struct _bdd_t
     /* Linux statistics */
     struct net_device_stats net_stats;
 
-    /* 82562EH variables */
-    uint16_t Phy82562EHSampleCount;
-    uint16_t Phy82562EHSampleDelay;
-    uint16_t Phy82562EHSampleFilter;
-
     uint16_t rfd_size;
 } bdd_t, *pbdd_t;
 
@@ -1960,7 +2049,10 @@ struct bdconfig
 #if	(DEBUG_LINK > 0)
 	uint16_t ErrorCounter; 
 #endif
-
+    
+    
+    uint8_t ifs_state;
+    uint8_t ifs_value;
 };
 
 #ifndef _BD_CONFIG_T_
@@ -1972,53 +2064,6 @@ typedef struct bdconfig bd_config_t;
 /*                                externs                                 */
 /* ====================================================================== */
 
-/* 82562EH PHY Specific Registers */
-#define PHY_82562EH_CONTROL_REG               0x10
-/* 82562EH control register bit definitions. */
-#define PHY_82562EH_CR_RAP            BIT_0
-#define PHY_82562EH_CR_HP             BIT_1   /* High power 1=HIGH 0=LOW */
-#define PHY_82562EH_CR_HS             BIT_2   /* High speed 1=HIGH 0=LOW */
-#define PHY_82562EH_CR_IRC            BIT_15  /* Ignore Remote Commands */
-
-/* 82562EH PHY address space constant define. */
-#define PHY_82562EH_ADDR_SPACE_PAGE_0     0
-#define PHY_82562EH_ADDR_SPACE_PAGE_1     1
-
-/* 82562EH PHY Registers, Page 0 */
-#define PHY_82562EH_STATUS_MODE_REG           0x11
-#define PHY_82562EH_NOISE_PEAK_LVL_REG        0x18
-#define PHY_82562EH_NSE_FLOOR_CEILING_REG     0x19
-#define PHY_82562EH_NSE_ATTACK_EVENT_REG      0x1A
-#define PHY_82562EH_RX_CONTROL_REG            0x1B
-
-/* 82562EH PHY Registers, Page 1 */
-#define PHY_82562EH_AFE_CONTROL1_REG          0x12
-#define PHY_82562EH_AFE_CONTROL2_REG          0x13
-
-/* 82562EH PHY Registers Initial Values, Page 0 */
-#define PHY_82562EH_STATUS_MODE_INIT_VALUE    0x0100  /* Reg 0x11 */
-#define PHY_82562EH_RX_CR_INIT_VALUE          0x0008  /* Reg 0x1B */
-
-/* 82562EH PHY Registers Initial Values, Page 1 */
-#define PHY_82562EH_AFE_CR1_INIT_VALUE        0x6600  /* Reg 0x12 */
-#define PHY_82562EH_AFE_CR2_INIT_VALUE        0x0100  /* Reg 0x11 */
-     
-/* 82562EH PHY S/W Init Values */
-#define PHY_82562EH_AFE_CR1_WA_INIT           0xA600  /* Page 1 Reg 0x12 */
-#define PHY_82562EH_AFE_CR2_WA_INIT           0x0900  /* Page 1 Reg 0x11 */
-#define PHY_82562EH_RX_CR_WA_INIT             0x0008  /* Page 0 Reg 0x1B */
-#define PHY_82562EH_STATUS_MODE_WA_INIT       0x0100  /* Page 0 Reg 0x11 */
-#define PHY_82562EH_NSE_ATTACK_EVENT_WA_INIT  0x0022  /* Page 0 Reg 0x1A */
-#define PHY_82562EH_NOISE_PEAK_LVL_WA_INIT    0x7F00  /* Page 0 Reg 0x19 */
-
-/* 82562EH PHY Working Mode Value */
-#define PHY_82562EH_NSE_ATTACK_WORK_VALUE         0xFFF4      /* Reg 0x1A */
-#define PHY_82562EH_AFE_CR1_WORK_VALUE            0x6600      /* Reg 0x12 */
-#define PHY_82562EH_AFE_CR2_WORK_VALUE            0x0100      /* Reg 0x11 */
-
-#define     MILLISECOND               1000
-#define     PHY_82562EH_MAX_TABLE     20
-
 
 /* ====================================================================== */
 /*                              vendor_info                               */
@@ -2950,3 +2995,5 @@ extern e100_vendor_info_t e100_vendor_info_array[];
 
 
 
+
+
